@@ -62,7 +62,7 @@ class SimpleCompletion(object):
     An object that hold all the information on how to retrieve possible completions in simple cases
     
     """
-    __metaclass__ = dbg.LogTheMethods
+    __metaclass__ = None if not dbg.debug else dbg.LogTheMethods
     autostart = False
     autostart_last_length = 0
     
@@ -134,14 +134,19 @@ class SimpleCompletion(object):
 
 
         self.master = master
-        self.master.connect('delete-event', self.destroy)
+        self._ids = {}
+        hid = self.master.connect('delete-event', self.destroy)
+        self._ids['master_delete_event'] = (master, hid)
         self.lookup_values = {}
         self.MATCH_OPERATOR = OP.get(self.master.metadata.bind.name, 'ILIKE')
         self.widget = widget
         self.field_name = field_name
         self.completion = self.prepare_completion()
         self.column = self.master.mapper_info.fields[field_name]['col']
-        self.table = self.column.table
+        try:
+            self.table = self.column.table
+        except AttributeError:
+            raise exc.ColumnWithoutTable("Column %s does not have .table attribute" % field_name)
         self.format = get_description(self.table, attr='format')
         self.description = tables[self.table.name].description
         self._attrs = tables[self.table.name].attrs
@@ -160,8 +165,16 @@ class SimpleCompletion(object):
         self.match_dict = {}
         
     def destroy(self, sqlwidget):
+
+        for obj, hid in self._ids.values():
+            obj.disconnect(hid)
+        del self._ids
+
         for attr in ('master', 'query', 'mapper', 'filtered_query', 'widget',):
-            delattr(self, attr)
+            try:
+                delattr(self, attr)
+            except AttributeError, e:
+                pass
         
     def set_group_by(self, field_name):
         """
@@ -262,7 +275,6 @@ class SimpleCompletion(object):
         :param pop:          don't try to pop the selection (debug purpose)
         :param one:          return the result that should be just one (used internally
                              within check_existance)
-        
         """
         self._apply_filter = apply_filter
         if self.force_enum:
@@ -272,7 +284,7 @@ class SimpleCompletion(object):
             # TIP: status bar 
             self.master.sb(_("search mode: %s") % mode)
         self.model.clear()
-        self._prev_gruop_by_value = None
+        self._prev_group_by_value = None
 
         result = self.select_values(mode, return_sql=return_sql, value=value)
         self._needs_recalculating = True
@@ -280,13 +292,34 @@ class SimpleCompletion(object):
             return debug_inline_params(result, bind=None)
 
         self.completion.set_model(None)
+        if not result:
+            try:
+                f = self.master.gui_fields[self.field_name]
+                q = self.filtered_query or self.query
+                result = q.filter_by(**{self.column_lookup.name : f.get_value()})
+            except exc.ValidationError:
+                # ValidationError occurs when f.get_value() fails finding a possible completion
+                return
+
         for obj in result:
             self.append_to_model(obj)
         self.completion.set_model(self.model)
 
-        if one and mode == 'exact':  # used by self.check_existance
-        ## just one result will be returned even if no check is done that more that one exists
-            return DictLike(obj)
+        if one and mode == 'exact':
+            ## 'exact' is used by self.check_existance just one result will
+            ## be returned even if no check is done that more that one exists
+
+            ## 'obj' may not be defined here. It happens when 1. you have a
+            ## filter on completion, 2. you bypass it with Control-Alt,
+            ## 3. select a match that activates the cell correctly. A second
+            ## pass on the same cell, if you try to complete, exact will be
+            ## used an will fail.
+            
+            try:
+                return DictLike(obj)
+            except UnboundLocalError:
+                return
+
         if pop:
             self.make_completion_to_pop(self.get_entry())
         else: # doctest purpose
@@ -347,10 +380,17 @@ class SimpleCompletion(object):
             else:
                 match = False
                 ## start mode
-                if self.mode == 'start' and re.search("^%s" %(key), text, re.IGNORECASE):
-                    match = True
-                elif re.search(key, text, re.IGNORECASE):  # regexp mode
-                    match = True
+                try:
+                    if not key:
+                        match = True
+                    elif text is None:
+                        match = False
+                    elif self.mode == 'start' and re.search("^%s" %(key), text, re.IGNORECASE):
+                        match = True
+                    elif re.search(key, text, re.IGNORECASE):  # regexp mode
+                        match = True
+                except re.error, e:
+                    match = False
                     
                 if match and self._group_by:
                     ## switch ON the display of group by label in completion
@@ -387,7 +427,9 @@ class SimpleCompletion(object):
         """
         self.displaying_completion = True
         entry.grab_focus()
-        entry.emit('changed')
+        # entry.emit('changed') ## needed to make completion to pop
+        # read note in widget.pop_completion()
+        self.widget.pop_completion()
         ## we don't want selection to prevent autocompletion from deleting already written text
         entry.select_region(-1,-1)
         
@@ -435,8 +477,9 @@ class SimpleCompletion(object):
             filtering_value = self.get_displayed_value(value=value)
             self.autostart_last_length = len(filtering_value)
 
-            query = self.compose_select_statement(query, oper, filtering_value)
-            query = self.add_dynamic_filters(query)
+            if filtering_value:
+                query = self.compose_select_statement(query, oper, filtering_value)
+        query = self.add_dynamic_filters(query)
 
         query = query.order_by(*self._order_by).distinct()
         if return_sql:
@@ -463,6 +506,10 @@ class SimpleCompletion(object):
     def set_values(self, values):
         """
         Set explicitely the possible completion values.
+
+        :param values: a list of values or a callable that will return a list
+                       of possible values
+
         Can be a list or a callable that will be called to get the
         real list of values passing the (possible) text in the entry
         as parameter
@@ -490,7 +537,7 @@ class SimpleCompletion(object):
             val = self.master.get_value(self.field_name, shown=True) or ''
         assert isinstance(val, basestring)
         self.autostart_last_length = len(val)
-        return self.add_for_LIKE_and_REGEXP(val)
+        return self.add_for_LIKE_and_REGEXP(val.strip('\\'))
 
     def get_match_operator(self):
         
@@ -714,6 +761,7 @@ class SimpleCompletion(object):
         Return True if field exists or validates
         """
         text = entry.get_text()
+        
         if not text:
             return True
 
@@ -729,13 +777,12 @@ class SimpleCompletion(object):
             try:
                 if self.master.is_fkey(self.field_name):
                     clean_value = self.master.gui_fields[self.field_name].clean_value(text, input_is_fkey=False)
-                    if not clean_value:
+                    if clean_value == None:
                         raise exc.NoResultFound
 
                     self.master.sb(_("Good match"), seconds=4)
-                    self.master.set_value(self.field_name, getattr(clean_value, self.column_lookup.name),
-                                          fkvalue=text)
-                    ## since on_completion needs an onj that must have all attributes defined in
+                    self.master.set_value(self.field_name, clean_value, fkvalue=text)
+                    ## since on_completion needs an obj that must have all attributes defined in
                     ## self.attrs we trigger the complete tour
                     obj = self.show_possible_completion(None, 'exact', one=True)
                     self.master.run_hook('on_completion', obj, field_name=self.field_name)
@@ -879,7 +926,7 @@ class FkeyCompletion(SimpleCompletion):
 
     def prepare_model(self):
         ## last col is for markup
-        return gtk.ListStore(str, str, str, object, bool, str)
+        return gtk.ListStore(object, str, str, object, bool, str)
             
     def compose_select_statement(self, query, oper, value):
 
@@ -891,7 +938,6 @@ class FkeyCompletion(SimpleCompletion):
                 query = query.filter(lookup_col.op(oper)(value))
         return query
     
-
     def match_selected_cb(self, wdg, model, iter, field_name):
         """
         after selecting an entry store the value in self.values and set
@@ -988,6 +1034,136 @@ class FkeyCompletion(SimpleCompletion):
 #         return compose_binary_expression(self.table, **kwargs)
 
 
+    def get_filtered_clause_list(self, **kw):
+        """
+        apply filter rules to this particular completion request.
+        Note: mapper is not available in the general situation, so we
+        use table to build the clause. That means we cannot follow any
+        further relationship.
+        """
+        
+        clause_list, path = django2sqlalchemy(table=self.table, **kw)
+        return clause_list
+
+    def __str__(self):
+        return "<FkeyCompletion: %s>" % self.field_name
+        
+class EnumCompletion(SimpleCompletion):
+    """
+    A completion that shows enum alternatives.
+    Inherits from :class:`SimpleCompletion`
+    """
+    VALUE_COL = 0
+    DESCRIPTION_COL = 1
+    SEARCH_COL = 1
+    MARKUP_COL = 1
+        
+    def __init__(self, master, widget, *args, **kw):
+        SimpleCompletion.__init__(self, master, widget, *args, **kw)
+        self.widget = widget
+        self._needs_recalculating = False
+    def get_entry(self):
+        return self.widget.entry
+
+    def prepare_model(self):
+        ## last col is for markup
+        return gtk.ListStore(object, str, )
+            
+    def match_selected_cb(self, wdg, model, iter, field_name):
+        """
+        after selecting an entry store the value in self.values and set
+        the entry with the correct value.
+        """
+        # the completion model has 1 or 4 columns (in case of FKey):
+        # 0:  the real value to be put in the db
+        # 1:  the value to be shown in the field
+        # 2:  the value to be shown in the drop down completion menu
+        # 3:  the row return by the SELECT when building the completion
+        #         if more fields are present more columns get written
+
+        value = model.get(iter, self.VALUE_COL)[0]
+        
+        fkvalue = model.get(iter, self.DESCRIPTION_COL)[0]
+
+        self.master.set_value(field_name, value, fkvalue=fkvalue, initial=False)
+
+        ## just handier to accept it already
+        if not self.master.is_mask():
+            self.master.cell_entry.entry.emit('activate')
+
+        self.displaying_completion = False
+        #self.master.run_hook('on_completion', obj, field_name=self.field_name)
+        return True
+
+### What to show
+    def show_possible_completion(self, widget, mode, apply_filter=True, pop=True, sb=True,
+                                 value=None, return_sql=False, one=False):
+        """
+        fill in a ListStore to use with completion, fetch data from db
+        
+        :param mode: determines the match of the already typed text:
+
+           :start: only matching data starting with typed text
+           :regexp: typed text is a regexp (*~ or ILIKE)
+           :enum: all values
+           :exact: triggers the completion machanism but just for the exact match
+
+        :param apply_filter: filter added with .filter() will be applied
+        :param return_sql:   boolean only the sql is returned (debug purpose)
+        :param value:        value to use as "displayed value". debug purpose
+        :param pop:          don't try to pop the selection (debug purpose)
+        :param one:          return the result that should be just one (used internally
+                             within check_existance)
+        """
+        self._apply_filter = apply_filter
+        if self.force_enum:
+            mode = 'enum'
+        self.mode = mode
+        if sb:
+            # TIP: status bar 
+            self.master.sb(_("search mode: %s") % mode)
+        self.model.clear()
+
+        self.completion.set_model(None)
+        field = self.master.gui_fields[self.field_name]
+
+        for idx, text in field.values.iteritems():
+            self.append_to_model(idx, text)
+        self.completion.set_model(self.model)
+
+        if one and mode == 'exact':  # used by self.check_existance
+        ## just one result will be returned even if no check is done that more that one exists
+            # I'm not sure that obj is *always* defined!... I believe so!
+            return DictLike(obj)
+
+        if pop:
+            self.make_completion_to_pop(self.get_entry())
+        else: # doctest purpose
+            for row in self.model:
+                print self.model.get_value(row.iter, 1)
+    def append_to_model(self, idx, text):
+        """
+        add to the model that has 6 cols
+        """
+        miter = self.model.append(None)
+        self.model.set(miter,
+                       self.VALUE_COL,       idx,
+                       self.DESCRIPTION_COL, text,
+                       )
+
+    def get_obj_no_flush(self, value):
+        """
+        check if the object exists and .one(), but don't trigger .flush()
+        """
+        
+        filter_cnd = {str(self.description) : value}
+        query = self.master.session.query(table2mapper(self.table)).autoflush(False)
+        return query.filter_by( **filter_cnd ).one()
+        
+#     def compose_binary_expression(self, **kwargs):
+#         return compose_binary_expression(self.table, **kwargs)
+
+
     def get_filtered_clause_list(self, **kw):
         """
         apply filter rules to this particular completion request.
