@@ -127,6 +127,7 @@ OP_MSGID = {
     'IS NOT FALSE' : _('The boolean is not False'),
     'IS NULL' : _('The value is not set'),
     'IS NOT NULL' : _('The value is set'),
+    'ID' : _("ID equality (don't follow foreign table)"),
 #    '' : _(''),
     }
 
@@ -154,8 +155,10 @@ class FilterPanel(object):
 
         """
         self.master = master
+        self._ids = {}
         self.dont_record_display = False
-        self.master.connect('delete-event', self.destroy)
+        handler_id = self.master.connect('delete-event', self.destroy)
+        self._ids['master_delete_event'] = (master, handler_id)
         self.get_layout(visible=visible)
         self.row = 0  # counter for position of fields
         self.default_reload = True
@@ -167,6 +170,11 @@ class FilterPanel(object):
         self._last_focused_entry = None
         if master.is_mask():
             self._id_record_selected = master.connect('record-selected', self.record_selected_cb)
+            h1 = self.master.connect('record-deleted', self.record_deleted_cb)
+            h2 = self.master.connect('record-new', self.record_added_cb)
+            self._ids['master_record_selected'] = (master, self._id_record_selected)
+            self._ids['master_record_deleted'] = (master, h1)
+            self._ids['master_record_new'] = (master, h2)
         self.prepare_actions()
 
     def record_selected_cb(self, mask):
@@ -175,6 +183,24 @@ class FilterPanel(object):
         """
         self.select_output_path(obj=mask.current, block=True)
         
+    def record_deleted_cb(self, mask, obj):
+        """
+        callback on record deletion
+        """
+        path = self.get_path_for_obj(obj)
+        if path is not None:
+            self.model.remove(self.model.get_iter(path) )
+        
+    def record_added_cb(self, mask):
+        """
+        callback on record deletion
+        """
+        try:
+            self.model.append([mask.current])
+        except:
+            ## FIXME: I should consider also  
+            self.model.append(None, [mask.current])
+        
     def get_layout(self, visible=True):
         """create the filter panel widget"""
         if self.master.is_mask():
@@ -197,7 +223,7 @@ class FilterPanel(object):
         self.l.prop('O.a', 'show_arrow', 'FALSE')
         self.l.prop('A=.fld', 'yscale', '0')
         self.l.elements['H.a'].pack_properties['y-options'] = ''
-        self.l.prop('O.a', 'toolbar_style', 'gtk.TOOLBAR_ICONS')
+        self.l.prop('O.a', 'toolbar_style', 'GTK_TOOLBAR_ICONS')
 
         if self.master.is_mask():
             ## TIP: filter page of the filter panel
@@ -401,6 +427,10 @@ class FilterPanel(object):
 
     def destroy(self, window=None):
 
+        for obj, hid in self._ids.values():
+            obj.disconnect(hid)
+        del self._ids
+
         if window:
             self.w['Window'].destroy()
         for attr in ('easytv', 'search_filter_widgets', 'l', 'model', 'selection', 'w'):
@@ -466,14 +496,16 @@ class FilterPanel(object):
             
     def get_path_for_obj(self, obj):
         """
-        return 
+        return path for object
+        :param obj: the object for which we neeed the path
         """
-        path_list = [0]
+        path_list = [None]
         def compare(model, path, iter):
-            if self.model.get_value(iter, 0) == obj:
-                
+            row_obj = model.get_value(iter, 0)
+            if row_obj == obj:
                 path_list[0] = path
                 return True
+            return False
         
         self.model.foreach(compare)
         return path_list[0]
@@ -530,11 +562,14 @@ class FilterPanel(object):
         view = columns.View(master=self.master, name='filter_panel', treeview=treeview,
                     field_list=field_list, ro=True)
         self.master.modelproxy = modelproxy.ModelProxy(self.master, treeview=treeview)
-        self.model = self.master.modelproxy.modelstore
         view.treeview.set_model(self.master.modelproxy.modelstore)
         self.master.views['filter_panel'] = view
         return view
-        
+    def _get_model(self):
+        return self.master.modelproxy.modelstore
+
+    model = property(_get_model)
+    
     def tv_resize(self):
         tv = self.w['TV=out']
         tv.realize()
@@ -733,10 +768,11 @@ class FilterTool(object):
             if len(self.master.relationship_path) == 1:
                 query = query.join( (self.table, self.master.relationship_path[0]))
             else:
-                ## NOTE: if relationship_path is longer that 1, the table is not aliased, i.e.: if you place
-                ## 2 search conditions to the same table (e.g. searching on staff and manager of a project-
-                ## and both manager and staff are m2m relation to User)
-                ## will most surely lead to no result
+                ## NOTE: if relationship_path is longer that 1, the table is
+                ## not aliased, i.e.: if you place 2 search conditions to
+                ## the same table (e.g. searching on staff and manager of a
+                ## project- and both manager and staff are m2m relation to
+                ## User) will most surely lead to no result
                 query = query.join(*self.master.relationship_path)
 
         if op_value in ('IS NULL', 'IS NOT NULL',
@@ -762,14 +798,16 @@ class FilterTool(object):
             try:
                 value = self.get_value()
             except exc.ValidationError, e:
-                msg = _("value '%s' cannot be used for field '%s'") % (self.get_row_value(), self.field_name)
+                msg = _("value '%s' cannot be used for field '%s'") % (
+                    self.get_row_value(), self.field_name)
                 self.master.sb(msg)
                 self.panel.sb(msg)
                 raise e
             
             if value not in (None, ''):
                 if not re.match('postgres|mysql', self.bind_name) and \
-                       op_value in ('ICONTAINS', 'MATCH'):
+                       op_value in ('ICONTAINS', 'MATCH') and \
+                       not isinstance(value, list): ## value is a list in FilterEnum 
                     op_value = 'ILIKE'
                     value = '%' + value + '%'
 
@@ -852,7 +890,7 @@ class FilterDateTool(FilterTool):
 
     def string_representation(self, value):
         """
-        when date algebra is used we dont need to do anything
+        when date algebra is used we don't need to do anything
         """
         if isinstance(value, date): 
             return FilterTool.string_representation(self, value)
@@ -872,13 +910,13 @@ class FilterFKeyTool(FilterTool):
     def get_suitable_operators(self):
 
         if self.bind_name.startswith('postgres'):
-            return  ['~*', '~', '!~*', '~*' ] + self.STD_OPERATORS
+            return  ['~*', '~', '!~*', '~*' ] + self.STD_OPERATORS + ['ID']
 
         elif self.bind_name.startswith('mysql'):
-            return ['REGEXP' ] + self.STD_OPERATORS
+            return ['REGEXP' ] + self.STD_OPERATORS + ['ID']
 
         else:
-            return ['ICONTAINS' ] + self.STD_OPERATORS
+            return ['ICONTAINS' ] + self.STD_OPERATORS + ['ID']
 
     def get_value(self):
         # no need to clean this value
@@ -896,6 +934,8 @@ class FilterFKeyTool(FilterTool):
         In case field is a fkey, and the destination lookup field is not in
         the mapper, it composes a subselect
         """
+        if op_value == 'ID':
+            return query.filter(self.col == value)
         fdescr = get_description(self.ftable, attr='description')
 
         if op_value == "ILIKE":
@@ -910,6 +950,52 @@ class FilterFKeyTool(FilterTool):
 
         return query.filter(self.col.in_(ids))
 
+class FilterEnumTool(FilterTool):
+
+    def __init__(self, field_name, panel, master):
+
+        FilterTool.__init__(self, field_name, panel, master)
+
+    def get_suitable_operators(self):
+
+        return ['~', '!~' ] + ['IS NULL', 'IS NOT NULL', 'ID']
+
+    def get_value(self):
+
+        op_value = self.get_operator()
+        
+        text = self.widget.get_value()
+        if not text or op_value == 'ID':
+            return text
+
+        field = self.master.gui_fields[self.field_name]
+        return [key for key, val in field.column.info['values']
+                  if re.search(text, val, re.IGNORECASE)]
+    
+    def string_representation(self, value):
+        """
+        a boolean must be always a string
+        """
+        return str(value)
+    
+    def _compose_field_whereclause(self, query, value, op_value):
+        """return a sql whereclause for field
+
+        In case field is a fkey, and the destination lookup field is not in
+        the mapper, it composes a subselect
+        """
+        from sqlalchemy.sql import not_
+        
+        if op_value == 'ID':
+            return query.filter(self.col == value)
+
+        elif op_value == '~':
+            return query.filter(self.col.in_(value))
+        
+        elif op_value == '!~':
+            return query.filter(not_(self.col.in_(value)))
+        
+        return query
 
 def get_filter_widget(field_name, panel, master, value=None):
     """
@@ -923,6 +1009,9 @@ def get_filter_widget(field_name, panel, master, value=None):
     elif master.is_fkey(field_name):
         return FilterFKeyTool(*args)
 
+    elif master.is_enum(field_name):
+        return FilterEnumTool(*args)
+    
     elif master.is_boolean(field_name):
         return FilterBooleanTool(*args)
     
@@ -979,7 +1068,8 @@ class FilterWidget(object):
         c = 0
         widgets = [ self.field_name_event_box, self.operator, self.check_button ]
         ftype  = self.tool.master.get_field_type(self.tool.field_name)
-        if not issubclass(ftype, bool):
+        
+        if self.tool.master.is_enum(self.tool.field_name) or not issubclass(ftype, bool):
             widgets += [self.field_widget]
             
         for wdg in widgets:
