@@ -20,6 +20,7 @@ import datetime
 import decimal
 import inspect
 import gc
+import shutil
 import warnings
 
 import gtk
@@ -37,7 +38,7 @@ from sqlkit import _, exc, layout, debug as  dbg, fields
 from sqlkit import layout, debug as  dbg
 from sqlkit.layout import map_labels, get_label
 from sqlkit.db import minspect, defaults
-from sqlkit.widgets.common import completion, sqlfilter, layoutgenerator
+from sqlkit.widgets.common import completion, sqlfilter, layoutgenerator, dialogs
 from sqlkit.misc.utils import Container, str2list
 
 NUMBERS = (int, float, decimal.Decimal)
@@ -59,6 +60,14 @@ class SqlWidget(gobject.GObject):
                            gobject.TYPE_BOOLEAN,
                            (),
                            ),
+        'record-new'   :   (gobject.SIGNAL_RUN_LAST,
+                           gobject.TYPE_BOOLEAN,
+                           (),
+                           ),
+        'record-deleted':  (gobject.SIGNAL_RUN_LAST,
+                           gobject.TYPE_BOOLEAN,
+                           (gobject.TYPE_PYOBJECT,),
+                           ),
         'records-displayed' : (gobject.SIGNAL_RUN_LAST,
                            gobject.TYPE_BOOLEAN,
                            (),
@@ -82,7 +91,7 @@ class SqlWidget(gobject.GObject):
     gui_fields = None
     values = None    # real value for foreign key
                      # (when lookup is displeyed instead)
-#    __metaclass__ = dbg.LogTheMethods
+    __metaclass__ = None if not dbg.debug else dbg.LogTheMethods
     records = None
     last_new_obj = None
 
@@ -274,6 +283,7 @@ class SqlWidget(gobject.GObject):
         """
 
         gobject.GObject.__init__(self)
+        self._ids = {}  # handler for connect to gobjects
         if what:
             if isinstance(what, (basestring, sqlalchemy.Table)):
                 table = what
@@ -338,10 +348,16 @@ class SqlWidget(gobject.GObject):
         self.relationship_path = self._get_relationship_path(relationship_path)
         self.relationship_leader = relationship_leader
         if relationship_leader:
-            relationship_leader.connect('delete-event', self.delete_event_cb)
+            hid = relationship_leader.connect('delete-event', self.delete_event_cb)
+            self._ids['relationship_leader'] = (relationship_leader, hid)
         self.records = []
         self.about_dialog = about_dialog or self.create_about_dialog()
-        self.show(show)
+        try:
+            self.show(show)
+        except exc.DBAPIError, e:
+            msg = _("Sorry, problems onnecting to remote db. Original error was: \n\n%s" % e)
+            raise exc.ConnectionError(msg)
+            
         if not relationship_leader:
             self._filter_panel =  sqlfilter.FilterPanel(self, visible=False)
         if hide_fields:
@@ -363,7 +379,6 @@ class SqlWidget(gobject.GObject):
         for extension in session.extensions:
             if isinstance(extension, SKSessionExtension): 
                 extension.sk_widgets = getattr(extension, 'sk_widgets', []) + [self]
-        self.connect('delete-event', self._clean_session_extensions)
         return session
 
     def _clean_session_extensions(self, *args):
@@ -395,6 +410,7 @@ class SqlWidget(gobject.GObject):
             mapper = self.dbproxy.get_mapper(mapper, tables)
             
         #self.tables = [mapper.local_table.name]  ## FIXME: what when we have a join?
+        mapper.compile()  # this insures that all attribute gets compile
         return mapper
 
     def _get_metadata(self, metadata, mapper):
@@ -748,7 +764,7 @@ class SqlWidget(gobject.GObject):
         Window = self.widgets['Window']
         w, h = Window.get_size()
 
-        if width <- 0:
+        if width <= 0:
             width = w
 
         if height  <= 0:
@@ -958,12 +974,12 @@ class SqlWidget(gobject.GObject):
         except (exc.DialogValidationError, exc.CancelledAction):
             return True
     
-    def record_save_new(self, ev):
+    def record_save_new_cb(self, action):
         """
         Save the record as a new record, ie letting pk not be set 
         """
-        raise NotImplementedError
-
+        self.record_save_new()
+        
     def record_add(self, record):
         """
         Add a record to the SqlWidget
@@ -986,6 +1002,7 @@ class SqlWidget(gobject.GObject):
 
             self.session.commit()
             self.emit('record-saved', )
+            self.run_hook('on_record_saved')
             self.sb(message, seconds=3)
 
         except (exceptions.DBAPIError, exceptions.FlushError), e:
@@ -1004,6 +1021,14 @@ class SqlWidget(gobject.GObject):
             self.message(type='error', text=_(msg))
             self.sb(str(e))
 
+        elif isinstance(e, exceptions.ProgrammingError):
+            # TIP: Error while saving into a table w/o permission
+            msg = _("A programming error was received from the backend:\n%s") % e
+            if re.search('permission denied', msg, re.I):
+                msg = "Access was denied while accessing one of the tables (%s)" % e
+            self.message(type='error', text=msg)
+            self.sb(e.orig)
+
         elif isinstance(e, exceptions.DBAPIError):
             # TIP: message in the status bar when a commit error is handled
             self.message(type='error',
@@ -1083,22 +1108,11 @@ class SqlWidget(gobject.GObject):
             if sqlwidget.record_has_changed():
                 return True
 
-        is_mask = self.is_mask()
         for field in self.gui_fields:      
             if not field.editable:
                 continue
             try:
-                if is_mask:
-                    ## this part is needed only for mask as in table the cell-renderer does it
-                    ## as soon as you leave the cell
-                    ## if we setattr now the value in the entry cannot reach the object
-                    ## My doubt is that we should probably firstly run the validation
-                    if not isinstance(field, fields.CollectionField):
-                        # CollectionField does not need to be set, it's already set to the list
-                        # and we need not keep the list to the related.records list
-                        setattr(obj, field.field_name, field.get_value())
-
-                if self.has_changed(field.field_name):
+                if field.has_changed():
                     return True
 
             except exc.ValidationError, e:
@@ -1372,7 +1386,6 @@ class SqlWidget(gobject.GObject):
         """
         Destroy the widget
         """
-        
         if 'Window' in self.widgets:
             ## This sqlwidget owns a toplevel Window. Destroying that one triggers
             ## delete_event_cb, that emits 'delete'event'
@@ -1380,11 +1393,16 @@ class SqlWidget(gobject.GObject):
             self.widgets['Window'].emit('delete-event', event)
         else:
             self.emit('delete-event')
-
+        
     def delete_event_cb(self, *args):
         """
         Quit operations: check if need to save first
         """
+        self._clean_session_extensions()
+        for obj, hid in self._ids.values():
+            obj.disconnect(hid)
+        del self._ids
+
         try:
             self.record_save()
         except (exc.DialogValidationError, exc.CancelledAction), e:
@@ -1439,6 +1457,8 @@ class SqlWidget(gobject.GObject):
             'mapper',
             'mapper_info',
             'metadata',
+            'modelproxy',
+            'last_new_obj',
             '_mode',
             'naked',
             'nick',
@@ -1460,11 +1480,13 @@ class SqlWidget(gobject.GObject):
             'sql',
 #             'tables',
             '_title',
+            '_filter_panel',
             'ui_browse_id',
             'ui_debug_id',
             'ui_manager',
             'ui_print_id',
             'widgets',
+            'views',
             'xml',
             ]
         for at in attrs:
@@ -1534,7 +1556,21 @@ class SqlWidget(gobject.GObject):
         if self.is_mask():
             self.current = None
             self.clear(check=False)
-        self.records = query.all()
+        try:
+            self.records = query.all()
+        except exceptions.ProgrammingError, e:
+            # TIP: reloading data from the database
+            msg = _("A programming error was received from the backend:\n%s") % e
+            if re.search('permission denied', msg, re.I):
+                msg = "Access was denied while accessing one of the tables (%s)" % e
+            self.message(type='error', text=msg)
+            self.sb(e.orig)
+            return
+        except exceptions.InterfaceError, e:
+            ## the connection may be closed and a retry could restore it
+            if re.search('closed', str(e), re.I):
+                self.records = query.all()
+            
         if display:
             self.record_display(check=False)
 
@@ -1825,7 +1861,10 @@ class SqlWidget(gobject.GObject):
         """
         Return true if this field is a String type
         """
-        return self.mapper_info.is_string(field_name)
+        try:
+            return self.mapper_info.is_string(field_name)
+        except AttributeError, e:
+            return isinstance(self.gui_fields[field_name], fields.VarcharField)
 
     def is_date(self, field_name):
         """
@@ -1883,7 +1922,7 @@ class SqlWidget(gobject.GObject):
 
     def is_numeric(self, field_name):
         """
-        Return true if this field is numeric
+        Return true if this field is numeric (rendered as Decimal)
         """
         try:
             return self.mapper_info.is_numeric(field_name)
@@ -1892,7 +1931,7 @@ class SqlWidget(gobject.GObject):
 
     def is_number(self, field_name):
         """
-        Return true if this field is numeric
+        Return true if this field is a number (int, float or Decimal)
         """
         try:
             return self.mapper_info.is_number(field_name)
@@ -1901,13 +1940,40 @@ class SqlWidget(gobject.GObject):
 
     def is_boolean(self, field_name):
         """
-        Return true if this field is  date
+        Return true if this field is boolean
         """
         try:
             return self.mapper_info.is_boolean(field_name)
         except AttributeError:
             return self.gui_fields[field_name].type == bool
 
+    def is_enum(self, field_name):
+        """
+        Return true if this field is to rendered as enum
+        """
+        try:
+            return self.mapper_info.is_enum(field_name)
+        except AttributeError:
+            return isinstance(self.gui_fields[field_name], fields.EnumField)
+
+    def is_image(self, field_name):
+        """
+        Return true if this field is image
+        """
+        try:
+            return self.mapper_info.is_image(field_name)
+        except AttributeError:
+            False
+
+    def is_file(self, field_name):
+        """
+        Return true if this field is image
+        """
+        try:
+            return self.mapper_info.is_file(field_name)
+        except AttributeError:
+            False
+
     def is_nullable(self, field_name):
         """
         Return true if this field is nullable
@@ -1946,7 +2012,10 @@ class SqlWidget(gobject.GObject):
                 return True
         except:
             pass
-        return self.mapper_info.is_text(field_name)
+        try:
+            return self.mapper_info.is_text(field_name)
+        except AttributeError:
+            return isinstance(self.gui_fields[field_name], fields.TextField)
 
     def is_toplevel(self):
         """
@@ -2188,7 +2257,7 @@ class UnsavedDataDialog(object):
                         
                     iter = self.model.append(None)
                     self.model.set(iter,
-                                   0, "<i>%s</i>" % (utils.clean_for_markup(repr(item))[:25] or
+                                   0, "<i>%s</i>" % (utils.clean_for_markup(repr(item)[:25]) or
                                                      utils.clean_for_markup(item)[:25]),
                                    1, self.clearer("%s" % self.master.get_label(field_name)),
                                    2, self.clearer(old_value),
@@ -2305,11 +2374,3 @@ class ValidationWarningDialog(ValidationErrorDialog):
             raise exc.DialogValidationError
         else:
             pass
-
-
-
-
-
-
-
-
