@@ -21,6 +21,7 @@ import gtk
 import gobject
 
 from sqlalchemy.exc import InvalidRequestError
+from sqlalchemy.orm.exc import ObjectDeletedError
 
 import sqlkit
 from sqlkit import debug as dbg, exc, _, layout, fields
@@ -37,7 +38,7 @@ class SqlMask(sqlwidget.SqlWidget):
     
     """
 
-    __metaclass__ = dbg.LogTheMethods
+    __metaclass__ = None if not dbg.debug else dbg.LogTheMethods
     labels   = None  # correspondance Label <-> EventBox
     row_count = 0    # number of records in the mapper
     current_idx = 0
@@ -255,10 +256,13 @@ class SqlMask(sqlwidget.SqlWidget):
 
         self.actiongroup_insert.add_actions([
             ('New', gtk.STOCK_NEW,   None, '', _('Add new record'), self.record_new_cb),
-            ('Save-as', gtk.STOCK_SAVE_AS, None, None, None, self.record_save_new),
+            ('Save-as', gtk.STOCK_SAVE_AS, None, None, None, self.record_save_new_cb),
             ])
         self.actiongroup_select.add_actions([
             ('Undo', gtk.STOCK_UNDO, None, None, _('Discard changes'), self.record_undo_cb),
+            # TIP Modify menu entry in the mask to reread a single record from database
+            ('RefreshRecord', gtk.STOCK_REFRESH, _('Refresh this record'), None,
+             _('Reread this record from db'), self.record_refresh_cb),
             ])
         self.actiongroup_delete.add_actions([
             ('Delete', gtk.STOCK_DELETE, None, '', _('Delete this record'), self.record_delete),
@@ -294,10 +298,13 @@ class SqlMask(sqlwidget.SqlWidget):
         if field_name not in self.gui_fields:
             raise exc.FieldNotInLayout("field_name: %s" % (field_name))
 
-        self.gui_fields[field_name].set_value(field_value, initial=initial, shown=shown)
+        field = self.gui_fields[field_name]
+        field.set_value(field_value, initial=initial, shown=shown)
 
+        
         if not initial:
-            self.run_hook('on_change_value', field_value, fkvalue, field_name=field_name)
+            fkvalue = fkvalue if fkvalue is not None else field.get_value(shown=True)
+            self.run_hook('on_change_value', field_value, fkvalue, field, field_name=field_name)
     
     def get_value(self, field_name, shown=False):
         """
@@ -324,7 +331,6 @@ class SqlMask(sqlwidget.SqlWidget):
 
         self.gui_fields[field_name].set_default()
         
-        
     def record_new(self, widget=None, clear=True):
         """
         prepare a new object and set
@@ -368,6 +374,7 @@ class SqlMask(sqlwidget.SqlWidget):
             r.records = getattr(self.current, key)
 
         self.current_mode = 'INSERT'
+        self.emit('record-new')
         
     def record_new_idle(self):
         """
@@ -385,6 +392,19 @@ class SqlMask(sqlwidget.SqlWidget):
         self.discard_changes()
         self.record_display(check=False)
         
+    def record_refresh_cb(self, menuItem):
+        """
+        undo possible modification to the mask
+        """
+        self.discard_changes()
+        self.session.expire(self.current)
+        try:
+            self.record_display(check=False)
+        except ObjectDeletedError:
+            # TIP message issued when a refresh is done on a deleted record
+            self.dialog(text=_('The record is no longer present in the database'))
+            self.record_display_next()
+
     def record_mask2obj(self, obj=None, force=False):
         """
         collect data from the mask and set into the object if record_has_changed())
@@ -444,8 +464,12 @@ class SqlMask(sqlwidget.SqlWidget):
         self.get_toplevel().set_focus(None)
 
         changed = self.record_has_changed()
-        if not new_obj and not changed and not (
-            self.unsaved_changes_exist(skip_new=self.current)):
+        if changed or new_obj or (
+            self.current in self.session.new and self.session.is_modified(self.current)
+            ) or (self.unsaved_changes_exist(skip_new=self.current)):
+            pass
+        
+        else:
             self.sb(_('Nothing to save'), seconds=4)
             return
 
@@ -472,9 +496,12 @@ class SqlMask(sqlwidget.SqlWidget):
         self.session.expire(self.current)
         self.record_display(check=False, delay_message=2)
         
-    def record_save_new(self, wdg, constraint={}):
+    def record_save_new(self, obj=None, commit=True, ask=True):
         """
-        create a new record by cloning the present
+        create a new record by duplicating the obj
+        :param obj: the obj to be cloned
+        :param commit: (boolean) commit after duplicating
+        :return: the new object
         """
 
         ## if PKey is editable we should add a check that is not the same
@@ -483,25 +510,72 @@ class SqlMask(sqlwidget.SqlWidget):
         else:
             tbl = self.tables
         pkey_changed = False
+
         for key in self.mapper_info.get_pkeys(tbl):
-            if key in self.gui_fields and self.is_editable(key):
-                if self.gui_fields[key].has_changed():
-                    pkey_changed = True
+            if key in self.gui_fields:
+                if self.is_editable(key):
+                    if self.gui_fields[key].has_changed():
+                        pkey_changed = True
+            else:
+                pkey_changed = True
                     
         if not pkey_changed:
             self.dialog(text=_("Primary key (%s) didn't change. Refusing to save as new" % key))
             return
-            
-        ## missing handling of Primary Key:
-        ## should I delete the old one? should I look for editable attribute of the
-        ## gtk widget?
 
+        o2m_list = [name for name, table in self.related.iteritems()
+                    if not table.relationship_mode == 'm2m']
+        save_as_is_managed = self.hooks and getattr(self.hooks, 'on_save_as', None)
+        if o2m_list and not save_as_is_managed:
+            msg = "Tables expressing one to many relationship will be left empty (%s)\n" + \
+                  "Read the docs for more informations"
+            response = self.dialog(type='ok-cancel', text=_(msg) % o2m_list,
+                                   icon='gtk-dialog-warning')
+            if response == gtk.RESPONSE_CANCEL:
+                return
+        elif ask:
+            msg = _("Do you want to copy all data in a new record?")
+            response = self.dialog(type='ok-cancel', text=_(msg) % o2m_list,
+                                   icon='gtk-dialog-question')
+            if response == gtk.RESPONSE_CANCEL:
+                return
+            
+        records_m2m = {}
+        for table_name, table in self.related.iteritems():
+            if table.relationship_mode == 'm2m':
+                records_m2m[table_name] = table.records
+
+        old_obj = self.current
         self.record_new(clear=False)
-        try:
-            self.record_save(ask=False, new_obj=True)
-        except exc.ValidationError, e:
-            pass
-        
+
+        for table_name, table in self.related.iteritems():
+            if table_name in records_m2m:
+                table.set_records(records_m2m[table_name])
+            else:
+                table.set_records([])
+
+        obj = obj or self.current
+        self.record_mask2obj(obj, force=True)
+        for table_name, table in self.related.iteritems():
+            setattr(obj, table_name, table.records)
+            
+        self.run_hook('on_save_as', old_obj, obj)
+
+        ## record_refresh() is not really needed here. You should implement it
+        #  in your own on_save_as, but you should know you need to...
+        for table in self.related:
+            if table.modelproxy.tree_field_name and not table.relationship_mode == 'm2m':
+                table.record_refresh()
+
+        if commit:
+            try:
+                self.record_save(ask=False)
+            except exc.ValidationError, e:
+                pass
+
+        self.emit('record-selected', )
+        return obj
+       
     def record_delete(self, widget=None, interactive=True):
         """
         delete a record
@@ -521,9 +595,15 @@ class SqlMask(sqlwidget.SqlWidget):
         
         try:
             self.commit(_('Deleted'))
+            self.emit('record-deleted', obj)
         except exc.HandledRollback, e:
             return
+        self.record_display_next()
             
+    def record_display_next(self):
+        """Display the next (or previous) record after deleting a record
+        """
+
         self.records.pop(self.current_idx)
 
         self.clear_mask(check=False)
