@@ -21,7 +21,7 @@ Each Column widget need a 'master' argument that is the SqlWidget
 in editing_started_cb and  edited_cb to set state variables on SqlWidget
 
 .. autoclass:: View
-   :members: add_column, select_path
+   :members: add_column, select_path, start_editing
 
 """
 
@@ -32,7 +32,7 @@ import pango
 from decimal import Decimal
 
 import cell_renderers, utils as tv_utils
-from sqlkit import debug as dbg, _, fields
+from sqlkit import debug as dbg, _, exc, fields
 from sqlkit.layout import widgets
 from sqlkit.misc.utils import str2list
 from sqlkit.db.utils import DictLike
@@ -59,7 +59,10 @@ class GenericColumn(object):
     EXPAND = True
     MIN_DEFAULT_WIDTH = 4
     MAX_DEFAULT_WIDTH = 30
+    PROPERTY = 'text'
     editable = False
+    __metaclass__ = None if not dbg.debug else dbg.LogTheMethods
+
     
     def __init__(self, master, field_name, title, field=None):
         """
@@ -79,8 +82,8 @@ class GenericColumn(object):
         self.field_name = field_name
         self.column = self.setup_column(title)
         cell = self.setup_cell_renderers()
-        self.column.set_cell_data_func(cell, self.cell_data_func_cb, 'text') 
-        self.column.cells_data_func = [(cell, self.cell_data_func_cb, 'text') ]
+        self.column.set_cell_data_func(cell, self.cell_data_func_cb, self.PROPERTY) 
+        self.column.cells_data_func = [(cell, self.cell_data_func_cb, self.PROPERTY) ]
 
     def setup_column(self, title):
         """
@@ -214,7 +217,7 @@ class GenericColumn(object):
 
     def on_activate(self, editable=None):
         # this is no longer connected to 'activate' signal since I prefere the changed value
-        # be available. I prefere to keep similarit with what happens in a normal
+        # be available. I prefere to keep similarity with what happens in a normal
         # entry where 'activate' means "press Enter". Pressing enter here make it edited.
         # That means it will also be triggered via Tab...
         self.master.run_hook('on_activate', self.field, field_name=self.field_name)
@@ -363,6 +366,7 @@ class BooleanColumn(GenericColumn):
         """
         callback that cicle into True/False/(inconsistent if nullable)
         """
+
         self.master.currently_edited_field_name = self.field_name
         self.master.initialize_record_editing(path)
 
@@ -377,7 +381,8 @@ class BooleanColumn(GenericColumn):
         else:
             status = not status
 
-        self.field.set_value(status, initial=False )        
+        #self.field.set_value(status, initial=False, obj=obj)
+        self.master.set_value(self.field_name, status, initial=False)
             
         return True
     
@@ -637,6 +642,142 @@ class DateTimeColumn(DateColumn):
         pass
         #editable.connect('key-press-event')
     
+class EnumColumn(VarcharColumn):
+
+    RENDERER = gtk.CellRendererText
+    RESIZE = True
+    REORDER = True
+    CLICK = True
+    EXPAND = False
+    MIN_DEFAULT_WIDTH = 4
+    MAX_DEFAULT_WIDTH = 30
+    PROPERTY = 'text'
+    editable = False
+
+    def cell_data_func_cb(self, column, cell, model, iter, prop_name):
+        """
+        cell_data_function used by column that have fkey
+        """
+
+        if not column.get_visible():
+            return
+
+        obj = model.get_value(iter, 0)
+        
+        if not obj:
+            return
+        
+        cell.set_property('visible', True)        
+
+        ## DB RECORDS
+        if isinstance(obj, self.master.mapper.class_):
+            value = getattr(obj, self.field_name)
+
+            if value is not None:
+                show_value = self.field.lookup_value(value)
+                cell.set_property('text', show_value)
+            else:
+                cell.set_property('text', '')
+            cell.set_property('editable', self.editable)
+
+        ## TOTALS
+        elif hasattr(self.master, 'totals') and isinstance(obj, self.master.totals.total_class):
+            cell.set_property('text', '')
+            cell.set_property('editable', False)        
+
+        ## HEADER
+        else:
+            cell.set_property('text', getattr(obj, self.field_name))
+            cell.set_property('editable', False)
+
+class ImageColumn(GenericColumn):
+
+    RENDERER = gtk.CellRendererPixbuf
+    RESIZE = True
+    REORDER = False
+    CLICK = True
+    EXPAND = False
+    MIN_DEFAULT_WIDTH = 4
+    MAX_DEFAULT_WIDTH = 30
+    PROPERTY = 'pixbuf'
+    editable = False
+
+    def set_editable(self, editable):
+        pass
+    def cell_set_property(self, cell):
+        pass
+
+    def setup_cell_renderers(self, cell_renderer=None):
+        """
+        setup cell renderers 
+        """
+
+        cell = cell_renderer or self.RENDERER()
+        self.column.pack_start(cell, True)
+        self.cell_set_property(cell)
+
+        return cell
+    
+    def cell_data_func_cb(self, column, cell, model, iter, prop_name):
+        """
+        function set by set_cell_data_func
+        gets the attribute from the field and render an image thumbnail
+        """
+
+        if not column.get_visible():
+            return
+        obj = model.get_value(iter, 0) 
+        ## when the column is not representing a db-column (eg. is a python function)
+        ## we pass the object itself to the function
+        if self.field.persistent:
+            value = getattr(obj, self.field_name)
+        else:
+            value = obj
+            if isinstance(obj, self.master.mapper.class_):
+                value = self.field.clean_value(value)
+
+        cell.set_property('visible', True)        
+
+        ## DB RECORDS
+        if isinstance(obj, self.master.mapper.class_):
+            cell.set_property('visible', True)
+            if value:
+                if self.field.is_image(value):
+                    thumbnail_path = value and self.field.get_thumbnail(value, complete_path=True) or None
+                    if thumbnail_path:
+                        if cell.get_data('thumbnail_path') != thumbnail_path:
+                            pixbuf = gtk.gdk.pixbuf_new_from_file(thumbnail_path) if thumbnail_path else None
+
+                            cell.set_property('pixbuf', pixbuf)
+                            cell.set_property('stock-id', None)
+                            cell.set_data('thumbnail_path', thumbnail_path)
+                    else:
+                        cell.set_property('pixbuf', None)
+                        cell.set_property('stock-id', gtk.STOCK_MISSING_IMAGE)
+                        cell.set_data('thumbnail_path', thumbnail_path)
+                        
+                else:
+                    cell.set_property('pixbuf', None)
+                    cell.set_property('stock-id', 'gtk-file')
+                    cell.set_data('thumbnail_path', None)
+            else:
+                cell.set_property(prop_name, None)
+                cell.set_data('thumbnail_path', None)
+                cell.set_property('stock-id', None)
+                
+
+        ## TOTALS
+        elif hasattr(self.master, 'totals') and isinstance(obj, self.master.totals.total_class):
+            cell.set_property(prop_name, None)
+            cell.set_data('thumbnail_path', None)
+            cell.set_property('stock-id', None)
+
+        ## HEADERS
+        else:
+            cell.set_property(prop_name, None)
+            cell.set_data('thumbnail_path', None)
+            cell.set_property('stock-id', None)
+    
 class CellEntry(object):
     """
     An object to store info on editable status
@@ -881,10 +1022,10 @@ class View(object):
 
     A View is a set of columns of the SqlTable's that offers a (possibly
     partial) view of the model. Each SqlTable has one mandatory view named
-    ``main`` and possibly other views. Attribute ``views`` (a
+    ``main`` and possibly other views. Attribute :attr:`sqlkit.widgets.SqlTable.views` (a
     sqlkit.utils.Container) stores them.
 
-    It can be used to provide different ways to look at the same data
+    It can be used to provide different way to look at the same data
     (e.g. different cell-renderers) or different columns for long ones
 
     """
@@ -1181,6 +1322,40 @@ class View(object):
         else:
             action_fk.set_visible(False)
             
+        action_upload_image = self.ui_manager.get_action('/TreePopup/UploadImage')
+        if obj and field_name and self.master.is_image(field_name):
+            action_upload_image.set_visible(True)
+        else:
+            action_upload_image.set_visible(False)
+            
+        action_new_child = self.ui_manager.get_action('/TreePopup/NewChild')
+        if self.master.modelproxy.is_tree():
+            new = self.ui_manager.get_action('/TreePopup/New').get_visible()
+            action_new_child.set_visible(new)
+        else:
+            action_new_child.set_visible(False)
+            
+
+        action_delete_file = self.ui_manager.get_action('/TreePopup/DeleteFile')
+        action_show_file = self.ui_manager.get_action('/TreePopup/ShowFile')
+        action_save_file = self.ui_manager.get_action('/TreePopup/SaveFile')
+        if obj and field_name and (self.master.is_image(field_name) or
+                               self.master.is_file(field_name)) and \
+                               getattr(obj, field_name):
+            action_delete_file.set_visible(True)
+            action_show_file.set_visible(True)
+            action_save_file.set_visible(True)
+        else:
+            action_delete_file.set_visible(False)
+            action_show_file.set_visible(False)
+            action_save_file.set_visible(False)
+            
+        action_upload_file = self.ui_manager.get_action('/TreePopup/UploadFile')
+        if field_name and self.master.is_file(field_name):
+            action_upload_file.set_visible(True)
+        else:
+            action_upload_file.set_visible(False)
+            
         action_delete = self.ui_manager.get_action('/TreePopup/RecordDelete')
         action_delete.set_sensitive(obj and True or False)
 
@@ -1192,9 +1367,10 @@ class View(object):
         
         return menu
 
-    def setup_columns(self, field_list=None):
+    def setup_columns(self, field_list=None, position=-1):
         """
         :param field_list: list of field_names or string to be splitted (comma or space separated)
+        :param position: determine the position in the columns where the column will be inserted
         """
 
         for field_name in field_list or self.field_list:
@@ -1211,6 +1387,12 @@ class View(object):
             elif self.master.is_fkey(field_name) :   ## FKey
                 col = FKeyColumn(self.master, field_name, label)
 
+            elif self.master.is_image(field_name):  ## image
+                col = ImageColumn(self.master, field_name, label)
+
+            elif self.master.is_enum(field_name):  ## image
+                col = EnumColumn(self.master, field_name, label)
+
             elif self.master.is_boolean(field_name):  ## boolean
                 col = BooleanColumn(self.master, field_name, label)
 
@@ -1232,17 +1414,27 @@ class View(object):
             else:
                 col = VarcharColumn(self.master, field_name, label)
 
-            self.add_column(col)
+            self.add_column(col, position=position)
             
         self.treeview.columns_autosize()
         self.fix_expandable_columns()
             
-    def add_column(self, column, position=-1):
+    def add_column(self, column=None, position=-1, field=None):
         """
-        Add a column to the view
+        Add a column to the view. 
 
-        :param column:
+        :param column: the column to be added. If empty, a field must be provided 
+        :param position: where should the column be inserted
+        :param field: an instance of the Field that should be used. In this case a
+              column is guessed and built on the fly by :meth:`setup_columns`
         """
+        if not column:
+            assert field is not None, \
+                   "You must pass a sqlkit.fields.Field instance"
+            self.master.gui_fields[field.field_name] = field
+            self.setup_columns([field.field_name], position=position)
+            return
+
         self.tvcolumns[column.field_name] = column.column
         self.columns[column.field_name] = column
         if position  < 0:
@@ -1261,16 +1453,32 @@ class View(object):
         second_choice = []
         for field_name in [c.get_data('field_name') for c in self.treeview.get_columns()]:
             if self.tvcolumns[field_name].get_visible() and self.master.is_editable(field_name):
-                if self.master.gui_fields[field_name].get_default():
+                try:
+                    has_usable_default = self.master.gui_fields[field_name].get_default()
+                except exc.NotHandledDefault, e:
+                    # it's not necessary to fill this value as a default exists even if we're
+                    # not able to handle it
+                    has_usable_default = True
+                if has_usable_default is not None:
                     second_choice += [self.tvcolumns[field_name]]
                 else:
                     return self.tvcolumns[field_name]
         if second_choice:
             return second_choice[0]
 
-    def start_editing(self, path):
+    def start_editing(self, path, field_name=None):
+        """
+        Edit a path/column. Column is passed by its field_name
 
-        col = self.get_first_editable_col()
+        :param path: the path to be edited
+        :param field_name: the field_name of the column, if None, the first editable field
+            that does not have a default
+        """
+
+        if field_name:
+            col = self.tvcolumns[field_name]
+        else:
+            col = self.get_first_editable_col()
         self.treeview.scroll_to_cell(path, col)
         self.treeview.set_cursor(path = path, focus_column = col,  start_editing = True)
         
@@ -1324,16 +1532,3 @@ class View(object):
 
         if path is not None:
             self.treeview.get_selection().select_path(path)
-            
-
-
-
-
-
-
-
-
-
-
-
-
